diff --git a/Makefile b/Makefile
index 17be480..5a6a8b5 100644
--- a/Makefile
+++ b/Makefile
@@ -29,12 +29,12 @@ ifeq ($(platform), unix)
 
 # use for raspberry pi
 else ifeq ($(platform), rpi)
-	   TARGET := $(TARGET_NAME)_libretro.so
-	   fpic := -fPIC
-		 LDFLAGS := -lpthread
-		 PLATFLAGS +=  -DARM  -marm
-	   SHARED := -shared -Wl,--version-script=$(CORE_DIR)/libretro/link.T
-
+	TARGET := $(TARGET_NAME)_libretro.so
+	fpic := -fPIC
+	LDFLAGS := -lpthread
+	PLATFLAGS +=  -DARM  -marm
+	SHARED := -shared -Wl,--version-script=$(CORE_DIR)/libretro/link.T -Wl,--as-needed -Wl,--no-undefined
+	LDFLAGS += -lm
 else ifeq ($(platform), osx)
    TARGET := $(TARGET_NAME)_libretro.dylib
    fpic := -fPIC -mmacosx-version-min=10.6
diff --git a/sources/src/filesys_linux.c b/sources/src/filesys_linux.c
index 613f138..87736da 100644
--- a/sources/src/filesys_linux.c
+++ b/sources/src/filesys_linux.c
@@ -9,9 +9,9 @@
 #include "zfile.h"
 
 #include <sys/time.h>
+#include <sys/timeb.h>
 
 #ifdef FTIME
-#include <sys/time.h>
 
 __BEGIN_DECLS
 
diff --git a/sources/src/inputdevice.c b/sources/src/inputdevice.c
index ba3b6bc..d0b8530 100644
--- a/sources/src/inputdevice.c
+++ b/sources/src/inputdevice.c
@@ -6645,7 +6645,9 @@ void setjoybuttonstate (int joy, int button, int state)
 		return;
 	}
 #endif
-	setbuttonstateall (&joysticks[joy], &joysticks2[joy], button, state ? 1 : 0);
+	if (joysticks) {
+		setbuttonstateall (&joysticks[joy], &joysticks2[joy], button, state ? 1 : 0);
+	}
 }
 
 /* buttonmask = 1 = normal toggle button, 0 = mouse wheel turn or similar
@@ -6672,6 +6674,8 @@ void setmousebuttonstateall (int mouse, uae_u32 buttonbits, uae_u32 buttonmask)
 	int i;
 	uae_u32 obuttonmask = mice2[mouse].buttonmask;
 
+	if (!mice)
+		return;
 	for (i = 0; i < ID_BUTTON_TOTAL; i++) {
 		if (buttonmask & (1 << i))
 			setbuttonstateall (&mice[mouse], &mice2[mouse], i, (buttonbits & (1 << i)) ? 1 : 0);
@@ -6689,6 +6693,8 @@ void setmousebuttonstate (int mouse, int button, int state)
 		inputdevice_testrecord (IDTYPE_MOUSE, mouse, IDEV_WIDGET_BUTTON, button, state, -1);
 		return;
 	}
+	if (!mice)
+		return;
 	setbuttonstateall (&mice[mouse], &mice2[mouse], button, state);
 	if (obuttonmask != mice2[mouse].buttonmask)
 		mousehack_helper (mice2[mouse].buttonmask);
@@ -6761,7 +6767,7 @@ void setmousestate (int mouse, int axis, int data, int isabs)
 		inputdevice_testrecord (IDTYPE_MOUSE, mouse, IDEV_WIDGET_AXIS, axis, 0, -1);
 		return;
 	}
-	if (input_play)
+	if (input_play || !mice)
 		return;
 	if (!mice[mouse].enabled) {
 		if (isabs && currprefs.input_tablet > 0) {
